@@ -273,7 +273,13 @@ desktop_thumbnailer_load_thumbnail (DesktopThumbnailer *thumbnailer,
   GdkPixbuf *source, *pixbuf = NULL;
   gboolean verbose;
 
-  g_object_get (G_OBJECT (thumbnailer), "exec", &exec, NULL);
+  if (path != NULL && g_str_has_prefix (path, g_get_tmp_dir ())
+      && g_regex_match_simple ("/tumbler-.{6}\\.png$", path, 0, 0))
+    {
+      g_set_error (error, TUMBLER_ERROR, TUMBLER_ERROR_IS_THUMBNAIL,
+                   TUMBLER_ERROR_MESSAGE_NO_THUMB_OF_THUMB, path);
+      return NULL;
+    }
 
   tmpfile = g_file_new_tmp ("tumbler-XXXXXX.png", &stream, error);
 
@@ -283,6 +289,7 @@ desktop_thumbnailer_load_thumbnail (DesktopThumbnailer *thumbnailer,
 
       size = MIN (width, height);
 
+      g_object_get (G_OBJECT (thumbnailer), "exec", &exec, NULL);
       res = desktop_thumbnailer_exec_parse (exec,
                                             path,
                                             uri,
@@ -294,7 +301,8 @@ desktop_thumbnailer_load_thumbnail (DesktopThumbnailer *thumbnailer,
 
       if (G_LIKELY (res))
         {
-          working_directory = g_path_get_dirname (path);
+          if (path != NULL)
+            working_directory = g_path_get_dirname (path);
 
           verbose = tumbler_util_is_debug_logging_enabled (G_LOG_DOMAIN);
           res = g_spawn_sync (working_directory,
@@ -331,10 +339,9 @@ desktop_thumbnailer_load_thumbnail (DesktopThumbnailer *thumbnailer,
       g_file_delete (tmpfile, NULL, NULL);
       g_object_unref (tmpfile);
       g_object_unref (stream);
+      g_free (exec);
     }
 
-  g_free (exec);
-
   return pixbuf;
 }
 
