@@ -23,8 +23,6 @@
 #include <config.h>
 #endif
 
-#include <math.h>
-
 #include <ft2build.h>
 #include FT_FREETYPE_H
 
@@ -174,12 +172,12 @@ render_glyph (GdkPixbuf *pixbuf,
   height = gdk_pixbuf_get_height (pixbuf);
   rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 
-  for (j = 0; j < slot->bitmap.rows; ++j)
+  for (j = 0; (guint) j < slot->bitmap.rows; ++j)
     {
       if (j + off_y < 0 || j + off_y >= height)
         continue;
 
-      for (i = 0; i < slot->bitmap.width; ++i)
+      for (i = 0; (guint) i < slot->bitmap.width; ++i)
         {
           if (i + off_x < 0 || i + off_x >= width)
             continue;
@@ -214,37 +212,6 @@ render_glyph (GdkPixbuf *pixbuf,
 
 
 
-static GdkPixbuf*
-scale_pixbuf (GdkPixbuf *source,
-              gint       dest_width,
-              gint       dest_height)
-{
-  gdouble wratio;
-  gdouble hratio;
-  gint    source_width;
-  gint    source_height;
-
-  /* determine source pixbuf dimensions */
-  source_width  = gdk_pixbuf_get_width  (source);
-  source_height = gdk_pixbuf_get_height (source);
-
-  /* determine which axis needs to be scaled down more */
-  wratio = (gdouble) source_width  / (gdouble) dest_width;
-  hratio = (gdouble) source_height / (gdouble) dest_height;
-
-  /* adjust the other axis */
-  if (hratio > wratio)
-    dest_width = rint (source_width / hratio);
-  else
-    dest_height = rint (source_height / wratio);
-
-  /* scale the pixbuf down to the desired size */
-  return gdk_pixbuf_scale_simple (source, MAX (dest_width, 1), MAX (dest_height, 1), 
-                                  GDK_INTERP_BILINEAR);
-}
-
-
-
 static GdkPixbuf *
 trim_and_scale_pixbuf (GdkPixbuf *pixbuf,
                        gint       dest_width,
@@ -358,7 +325,7 @@ trim_and_scale_pixbuf (GdkPixbuf *pixbuf,
   if (gdk_pixbuf_get_width (subpixbuf) > dest_width 
       || gdk_pixbuf_get_height (subpixbuf) > dest_height)
     {
-      scaled = scale_pixbuf (subpixbuf, dest_width, dest_height);
+      scaled = tumbler_util_scale_pixbuf (subpixbuf, dest_width, dest_height);
       g_object_unref (G_OBJECT (subpixbuf));
       subpixbuf = scaled;
     }
@@ -438,7 +405,6 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   GdkPixbuf              *pixbuf;
   FT_Error                ft_error;
   FT_Face                 face;
-  guint64                 mtime;
   GError                 *error = NULL;
   GFile                  *file;
   gchar                  *error_msg;
@@ -455,6 +421,7 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
     return;
 
   uri = tumbler_file_info_get_uri (info);
+  g_debug ("Handling URI '%s'", uri);
 
   /* check if we have a valid freetype library object */
   if (font_thumbnailer->library_error != 0)
@@ -462,7 +429,8 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
       /* there was an error in the freetype initialization, abort */
       error_msg = g_strdup_printf (_("Could not initialize freetype: %s"),
                                    ft_strerror (font_thumbnailer->library_error));
-      g_signal_emit_by_name (thumbnailer, "error", uri, 0, error_msg);
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             TUMBLER_ERROR, TUMBLER_ERROR_UNSUPPORTED, error_msg);
       g_free (error_msg);
       return;
     }
@@ -474,7 +442,8 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
       /* there was an error, emit error signal */
       error_msg = g_strdup_printf (_("Could not load file contents: %s"), 
                                    error->message);
-      g_signal_emit_by_name (thumbnailer, "error", uri, 0, error_msg);
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             error->domain, error->code, error_msg);
       g_free (error_msg);
 
       /* clean up */
@@ -493,7 +462,8 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
       /* the font file could not be loaded, emit an error signal */
       error_msg = g_strdup_printf (_("Could not open font file: %s"),
                                    ft_strerror (ft_error));
-      g_signal_emit_by_name (thumbnailer, "error", uri, 0, error_msg);
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             TUMBLER_ERROR, TUMBLER_ERROR_NO_CONTENT, error_msg);
       g_free (error_msg);
 
       /* clean up */
@@ -512,12 +482,13 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
         {
           /* try to set the character map */
           ft_error = FT_Set_Charmap (face, face->charmaps[n]);
-          if (G_UNLIKELY (error != 0))
+          if (G_UNLIKELY (ft_error != 0))
             {
               /* emit an error signal */
               error_msg = g_strdup_printf (_("Could not set the character map: %s"),
                                            ft_strerror (ft_error));
-              g_signal_emit_by_name (thumbnailer, "error", uri, 0, error_msg);
+              g_signal_emit_by_name (thumbnailer, "error", uri,
+                                     TUMBLER_ERROR, TUMBLER_ERROR_INVALID_FORMAT, error_msg);
               g_free (error_msg);
 
               /* clean up */
@@ -544,7 +515,8 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
       /* emit an error signal */
       error_msg = g_strdup_printf (_("Could not render glyphs: %s"),
                                    ft_strerror (ft_error));
-      g_signal_emit_by_name (thumbnailer, "error", uri, 0, error_msg);
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             TUMBLER_ERROR, TUMBLER_ERROR_INVALID_FORMAT, error_msg);
       g_free (error_msg);
 
       /* clean up */
@@ -557,9 +529,6 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   /* release the font face */
   FT_Done_Face (face);
 
-  /* determine when the URI was last modified */
-  mtime = tumbler_file_info_get_mtime (info);
-
   /* compose the image data */
   data.data = gdk_pixbuf_get_pixels (pixbuf);
   data.has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
@@ -570,13 +539,16 @@ font_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   data.colorspace = (TumblerColorspace) gdk_pixbuf_get_colorspace (pixbuf);
 
   /* save the thumbnail */
-  tumbler_thumbnail_save_image_data (thumbnail, &data, mtime, NULL, &error);
+  tumbler_thumbnail_save_image_data (thumbnail, &data,
+                                     tumbler_file_info_get_mtime (info),
+                                     NULL, &error);
 
   /* check if there was an error */
   if (error != NULL)
     {
       /* emit an error signal */
-      g_signal_emit_by_name (thumbnailer, "error", uri, error->code, error->message);
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             error->domain, error->code, error->message);
       g_error_free (error);
     }
   else
