@@ -55,8 +55,8 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (PopplerThumbnailerProvider,
                                 poppler_thumbnailer_provider,
                                 G_TYPE_OBJECT,
                                 0,
-                                TUMBLER_ADD_INTERFACE (TUMBLER_TYPE_THUMBNAILER_PROVIDER,
-                                                       poppler_thumbnailer_provider_thumbnailer_provider_init));
+                                G_IMPLEMENT_INTERFACE_DYNAMIC (TUMBLER_TYPE_THUMBNAILER_PROVIDER,
+                                                               poppler_thumbnailer_provider_thumbnailer_provider_init));
 
 
 
@@ -121,6 +121,10 @@ poppler_thumbnailer_provider_get_thumbnailers (TumblerThumbnailerProvider *provi
   /* add the thumbnailer to the list */
   thumbnailers = g_list_append (thumbnailers, thumbnailer);
 
+  tumbler_util_dump_strv (G_LOG_DOMAIN, "Supported URI schemes",
+                          (const gchar *const *) uri_schemes);
+  tumbler_util_dump_strv (G_LOG_DOMAIN, "Supported mime types", mime_types);
+
   /* free URI schemes and MIME types */
   g_strfreev (uri_schemes);
 
