@@ -9,11 +9,11 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General 
- * Public License along with this library; if not, write to the 
+ * You should have received a copy of the GNU Library General
+ * Public License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
@@ -22,8 +22,6 @@
 #include <config.h>
 #endif
 
-#include <math.h>
-
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
@@ -56,7 +54,7 @@ struct _PopplerThumbnailer
 
 
 
-G_DEFINE_DYNAMIC_TYPE (PopplerThumbnailer, 
+G_DEFINE_DYNAMIC_TYPE (PopplerThumbnailer,
                        poppler_thumbnailer,
                        TUMBLER_TYPE_ABSTRACT_THUMBNAILER);
 
@@ -96,41 +94,121 @@ poppler_thumbnailer_init (PopplerThumbnailer *thumbnailer)
 
 
 static GdkPixbuf *
-generate_pixbuf (GdkPixbuf              *source,
-                 TumblerThumbnailFlavor *flavor)
+poppler_thumbnailer_pixbuf_from_surface (cairo_surface_t *surface)
 {
-  gdouble    hratio;
-  gdouble    wratio;
-  gint       dest_width;
-  gint       dest_height;
-  gint       source_width;
-  gint       source_height;
+#if 0
+  return gdk_pixbuf_get_from_surface (surface,
+                                      0, 0,
+                                      cairo_image_surface_get_width (surface),
+                                      cairo_image_surface_get_height (surface));
+#else
+  GdkPixbuf       *pixbuf;
+  cairo_surface_t *image;
+  cairo_t         *cr;
+  gboolean         has_alpha;
+  gint             width, height;
+  cairo_format_t   surface_format;
+  gint             pixbuf_n_channels;
+  gint             pixbuf_rowstride;
+  guchar          *pixbuf_pixels;
+  gint             x, y;
+  guchar          *p;
+  guchar           tmp;
 
-  /* determine the source pixbuf dimensions */
-  source_width = gdk_pixbuf_get_width (source);
-  source_height = gdk_pixbuf_get_height (source);
+  width = cairo_image_surface_get_width (surface);
+  height = cairo_image_surface_get_height (surface);
 
-  /* determine the desired size for this flavor */
-  tumbler_thumbnail_flavor_get_size (flavor, &dest_width, &dest_height);
+  surface_format = cairo_image_surface_get_format (surface);
+  has_alpha = (surface_format == CAIRO_FORMAT_ARGB32);
 
-  /* return the same pixbuf if no scaling is required */
-  if (source_width <= dest_width && source_height <= dest_height)
-    return g_object_ref (source);
+  pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width, height);
+  pixbuf_n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+  pixbuf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+  pixbuf_pixels = gdk_pixbuf_get_pixels (pixbuf);
 
-  /* determine which axis needs to be scaled down more */
-  wratio = (gdouble) source_width / (gdouble) dest_width;
-  hratio = (gdouble) source_height / (gdouble) dest_height;
+  image = cairo_image_surface_create_for_data (pixbuf_pixels,
+                                               surface_format,
+                                               width, height,
+                                               pixbuf_rowstride);
+  cr = cairo_create (image);
+  cairo_set_source_surface (cr, surface, 0, 0);
 
-  /* adjust the other axis */
-  if (hratio > wratio)
-    dest_width = rint (source_width / hratio);
+  if (has_alpha)
+    cairo_mask_surface (cr, surface, 0, 0);
   else
-    dest_height = rint (source_height / wratio);
-  
-  /* scale the pixbuf down to the desired size */
-  return gdk_pixbuf_scale_simple (source, 
-                                  MAX (dest_width, 1), MAX (dest_height, 1), 
-                                  GDK_INTERP_BILINEAR);
+    cairo_paint (cr);
+
+  cairo_destroy (cr);
+  cairo_surface_destroy (image);
+
+  for (y = 0; y < height; y++)
+    {
+      p = pixbuf_pixels + y * pixbuf_rowstride;
+
+      for (x = 0; x < width; x++)
+        {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+          tmp = p[0];
+          p[0] = p[2];
+          p[2] = tmp;
+          p[3] = has_alpha ? p[3] : 0xff;
+#else
+          tmp = p[0];
+          p[0] = p[1];
+          p[1] = p[2];
+          p[2] = p[3];
+          p[3] = has_alpha ? tmp : 0xff;
+#endif
+          p += pixbuf_n_channels;
+        }
+    }
+
+  return pixbuf;
+#endif
+}
+
+
+
+static GdkPixbuf *
+poppler_thumbnailer_pixbuf_from_page (PopplerPage *page)
+{
+  cairo_surface_t *surface;
+  cairo_t         *cr;
+  GdkPixbuf       *pixbuf;
+  gdouble          width, height;
+
+  /* get the page size */
+  poppler_page_get_size (page, &width, &height);
+
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
+  cr = cairo_create (surface);
+
+  cairo_save (cr);
+  poppler_page_render (page, cr);
+  cairo_restore (cr);
+
+  cairo_set_operator (cr, CAIRO_OPERATOR_DEST_OVER);
+  cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+  cairo_paint (cr);
+
+  cairo_destroy (cr);
+
+  pixbuf = poppler_thumbnailer_pixbuf_from_surface (surface);
+
+  cairo_surface_destroy (surface);
+
+  return pixbuf;
+}
+
+
+
+/* to be removed when Poppler version >= 0.82: GBytes takes care of that */
+static void
+poppler_thumbnailer_free (gpointer data)
+{
+#if ! POPPLER_CHECK_VERSION (0, 82, 0)
+  g_free (data);
+#endif
 }
 
 
@@ -146,28 +224,31 @@ poppler_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   PopplerDocument        *document;
   PopplerPage            *page;
   const gchar            *uri;
+  cairo_surface_t        *surface;
   GdkPixbuf              *source_pixbuf;
   GdkPixbuf              *pixbuf;
   GError                 *error = NULL;
-  gdouble                 page_width;
-  gdouble                 page_height;
   GFile                  *file;
+#if POPPLER_CHECK_VERSION (0, 82, 0)
+  GBytes                 *bytes;
+#endif
   gchar                  *contents = NULL;
   gsize                   length;
+  gint                    width, height;
 
   g_return_if_fail (IS_POPPLER_THUMBNAILER (thumbnailer));
   g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
   g_return_if_fail (TUMBLER_IS_FILE_INFO (info));
 
   /* do nothing if cancelled */
-  if (g_cancellable_is_cancelled (cancellable)) 
+  if (g_cancellable_is_cancelled (cancellable))
     return;
 
-  /* try to load the PDF/PS file based on the URI */
   uri = tumbler_file_info_get_uri (info);
-  document = poppler_document_new_from_file (uri, NULL, &error);
+  g_debug ("Handling URI '%s'", uri);
 
-  /* check if that failed */
+  /* try to load the PDF/PS file based on the URI */
+  document = poppler_document_new_from_file (uri, NULL, &error);
   if (document == NULL)
     {
       /* make sure to free error data */
@@ -178,8 +259,8 @@ poppler_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
       /* try to load the file contents using GIO */
       if (!g_file_load_contents (file, cancellable, &contents, &length, NULL, &error))
         {
-          g_signal_emit_by_name (thumbnailer, "error", uri, TUMBLER_ERROR_UNSUPPORTED, 
-                                 error->message);
+          g_signal_emit_by_name (thumbnailer, "error", uri,
+                                 error->domain, error->code, error->message);
           g_error_free (error);
           g_object_unref (file);
           return;
@@ -189,26 +270,33 @@ poppler_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
       g_object_unref (file);
 
       /* try to create a poppler document based on the file contents */
+#if POPPLER_CHECK_VERSION (0, 82, 0)
+      bytes = g_bytes_new_take (contents, length);
+      document = poppler_document_new_from_bytes (bytes, NULL, &error);
+      g_bytes_unref (bytes);
+#else
       document = poppler_document_new_from_data (contents, length, NULL, &error);
+#endif
     }
 
   /* emit an error if both ways to load the document failed */
   if (document == NULL)
     {
-      g_signal_emit_by_name (thumbnailer, "error", uri, TUMBLER_ERROR_INVALID_FORMAT, 
-                             error->message);
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             error->domain, error->code, error->message);
       g_error_free (error);
-      g_free (contents);
+      poppler_thumbnailer_free (contents);
       return;
     }
 
   /* check if the document has content (= at least one page) */
   if (poppler_document_get_n_pages (document) <= 0)
     {
-      g_signal_emit_by_name (thumbnailer, "error", uri, TUMBLER_ERROR_NO_CONTENT, 
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             TUMBLER_ERROR, TUMBLER_ERROR_NO_CONTENT,
                              _("The document is empty"));
       g_object_unref (document);
-      g_free (contents);
+      poppler_thumbnailer_free (contents);
       return;
     }
 
@@ -217,10 +305,11 @@ poppler_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
 
   if (page == NULL)
     {
-      g_signal_emit_by_name (thumbnailer, "error", uri, TUMBLER_ERROR_NO_CONTENT,
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             TUMBLER_ERROR, TUMBLER_ERROR_NO_CONTENT,
                              _("First page of the document could not be read"));
       g_object_unref (document);
-      g_free (contents);
+      poppler_thumbnailer_free (contents);
       return;
     }
 
@@ -231,14 +320,16 @@ poppler_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   flavor = tumbler_thumbnail_get_flavor (thumbnail);
 
   /* try to extract the embedded thumbnail */
-  source_pixbuf = poppler_page_get_thumbnail_pixbuf (page);
-
-  if (source_pixbuf == NULL)
+  surface = poppler_page_get_thumbnail (page);
+  if (surface != NULL)
+    {
+      source_pixbuf = poppler_thumbnailer_pixbuf_from_surface (surface);
+      cairo_surface_destroy (surface);
+    }
+  else
     {
       /* fall back to rendering the page ourselves */
-      poppler_page_get_size (page, &page_width, &page_height);
-      source_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, page_width, page_height);
-      poppler_page_render_to_pixbuf (page, 0, 0, page_width, page_height, 1.0, 0, source_pixbuf);
+      source_pixbuf = poppler_thumbnailer_pixbuf_from_page (page);
     }
 
   /* release allocated poppler data */
@@ -246,7 +337,8 @@ poppler_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   g_object_unref (document);
 
   /* generate the final pixbuf (involves rescaling etc.) */
-  pixbuf = generate_pixbuf (source_pixbuf, flavor);
+  tumbler_thumbnail_flavor_get_size (flavor, &width, &height);
+  pixbuf = tumbler_util_scale_pixbuf (source_pixbuf, width, height);
   g_object_unref (flavor);
 
   g_assert (pixbuf != NULL);
@@ -259,13 +351,14 @@ poppler_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   data.rowstride = gdk_pixbuf_get_rowstride (pixbuf);
   data.colorspace = (TumblerColorspace) gdk_pixbuf_get_colorspace (pixbuf);
 
-  tumbler_thumbnail_save_image_data (thumbnail, &data, 
-                                     tumbler_file_info_get_mtime (info), 
+  tumbler_thumbnail_save_image_data (thumbnail, &data,
+                                     tumbler_file_info_get_mtime (info),
                                      NULL, &error);
 
   if (error != NULL)
     {
-      g_signal_emit_by_name (thumbnailer, "error", uri, error->code, error->message);
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             error->domain, error->code, error->message);
       g_error_free (error);
     }
   else
@@ -277,5 +370,5 @@ poppler_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   g_object_unref (thumbnail);
   g_object_unref (pixbuf);
   g_object_unref (source_pixbuf);
-  g_free (contents);
+  poppler_thumbnailer_free (contents);
 }
