@@ -23,6 +23,7 @@
 #endif
 
 #include <stdlib.h>
+#include <math.h>
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -391,7 +392,7 @@ xdg_cache_thumbnail_save_image_data (TumblerThumbnail *thumbnail,
     {
       /* convert the modified time of the source URI to a string */
       mtime_str = g_strdup_printf ("%" G_GUINT64_FORMAT ".%.6" G_GUINT32_FORMAT,
-                                   mtime_int, (guint32) (1.e6 * (mtime - mtime_int)));
+                                   mtime_int, (guint32) round (1.e6 * (mtime - mtime_int)));
 
       /* try to save the pixbuf */
       if (gdk_pixbuf_save_to_stream (dest_pixbuf, G_OUTPUT_STREAM (stream), "png",
