@@ -56,8 +56,6 @@ enum
   IOPRIO_CLASS_IDLE,
 };
 
-
-
 enum 
 {
   IOPRIO_WHO_PROCESS = 1,
@@ -75,39 +73,18 @@ enum
   LAST_SIGNAL,
 };
 
-
-static void tumbler_scheduler_class_init (TumblerSchedulerIface *klass);
 static guint tumbler_scheduler_signals[LAST_SIGNAL];
 
-GType
-tumbler_scheduler_get_type (void)
-{
-  static volatile gsize g_define_type_id__volatile = 0;
-  
-  if (g_once_init_enter (&g_define_type_id__volatile))
-    {
-      GType g_define_type_id =
-        g_type_register_static_simple (G_TYPE_INTERFACE,
-                                       "TumblerScheduler",
-                                       sizeof (TumblerSchedulerIface),
-                                       (GClassInitFunc) tumbler_scheduler_class_init,
-                                       0,
-                                       NULL,
-                                       0);
 
-      g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_OBJECT);
 
-      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
-    }
+G_DEFINE_INTERFACE (TumblerScheduler, tumbler_scheduler, G_TYPE_OBJECT)
 
-  return g_define_type_id__volatile;
-}
 
 
 static void
-tumbler_scheduler_class_init (TumblerSchedulerIface *klass)
+tumbler_scheduler_default_init (TumblerSchedulerIface *klass)
 {
-  g_object_interface_install_property (klass, 
+  g_object_interface_install_property (klass,
                                        g_param_spec_string ("name",
                                                             "name",
                                                             "name",
@@ -122,11 +99,12 @@ tumbler_scheduler_class_init (TumblerSchedulerIface *klass)
                   G_STRUCT_OFFSET (TumblerSchedulerIface, error),
                   NULL,
                   NULL,
-                  tumbler_marshal_VOID__UINT_BOXED_INT_STRING_STRING,
+                  tumbler_marshal_VOID__UINT_BOXED_UINT_INT_STRING_STRING,
                   G_TYPE_NONE,
-                  5,
+                  6,
                   G_TYPE_UINT,
                   G_TYPE_STRV,
+                  G_TYPE_UINT,
                   G_TYPE_INT,
                   G_TYPE_STRING,
                   G_TYPE_STRING);
@@ -249,14 +227,14 @@ tumbler_scheduler_emit_uri_error (TumblerScheduler        *scheduler,
   g_return_if_fail (error != NULL);
 
   g_signal_emit (scheduler, tumbler_scheduler_signals[SIGNAL_ERROR], 0, request->handle,
-                 uris, error->code, error->message, request->origin);
+                 uris, error->domain, error->code, error->message, request->origin);
 }
 
 
 
 TumblerSchedulerRequest *
 tumbler_scheduler_request_new (TumblerFileInfo    **infos,
-                               TumblerThumbnailer **thumbnailers,
+                               GList              **thumbnailers,
                                guint                length,
                                const gchar         *origin)
 {
@@ -292,9 +270,10 @@ tumbler_scheduler_request_new (TumblerFileInfo    **infos,
 
 
 void
-tumbler_scheduler_request_free (TumblerSchedulerRequest *request)
+tumbler_scheduler_request_free (gpointer data)
 {
-  gint n;
+  TumblerSchedulerRequest *request = data;
+  gint                     n;
 
   g_return_if_fail (request != NULL);
 
@@ -356,3 +335,37 @@ tumbler_scheduler_thread_use_lower_priority (void)
     sched_setscheduler (0, SCHED_IDLE, &sp);
 #endif
 }
+
+
+
+void
+tumbler_scheduler_thumberr_debuglog (TumblerThumbnailer      *thumbnailer,
+                                     const gchar             *failed_uri,
+                                     GQuark                   error_domain,
+                                     gint                     error_code,
+                                     const gchar             *message,
+                                     TumblerSchedulerRequest *request)
+{
+  gchar *text;
+
+  g_return_if_fail (TUMBLER_IS_THUMBNAILER (thumbnailer));
+  g_return_if_fail (failed_uri != NULL);
+  g_return_if_fail (request != NULL);
+  g_return_if_fail (TUMBLER_IS_SCHEDULER (request->scheduler));
+
+  if (error_domain == G_IO_ERROR && error_code == G_IO_ERROR_CANCELLED)
+    return;
+
+  if (error_domain == TUMBLER_ERROR)
+    text = g_strdup (message);
+  else
+    {
+      error_code = TUMBLER_ERROR_OTHER_ERROR_DOMAIN;
+      text = g_strdup_printf ("(domain %s, code %d) %s",
+                              g_quark_to_string (error_domain), error_code, message);
+    }
+
+  g_debug ("Failed attempt for job %d, URI '%s': Code %d, message: %s",
+           request->handle, failed_uri, error_code, text);
+  g_free (text);
+}
